<?php

namespace Drupal\drd\Plugin\Action;

use Drupal\drd\HttpRequest;
use Drupal\drd\Entity\DomainInterface;
use Drupal\drd\Entity\BaseInterface;

/**
 * Base class for DRD Remote Action plugins.
 */
abstract class BaseEntityRemote extends BaseEntity {

  protected $response = FALSE;
  protected $responseHeaders = [];

  /** @var \Drupal\drd\Crypt\BaseMethodInterface */
  protected $crypt;

  /**
   * @param string $id
   * @param BaseInterface $remote
   *
   * @return array|bool
   */
  public static function response($id, $remote, $arguments = []) {
    /** @var BaseEntityInterface $action */
    $action = self::instance($id);
    if ($action) {
      foreach ($arguments as $key => $value) {
        $action->setActionArgument($key, $value);
      }
      return $action->executeAction($remote);
    }
    return FALSE;
  }

  /**
   * Default remote action which can be overwritten by implementations.
   *
   * @param DomainInterface $domain
   * @return array|bool
   */
  public function executeAction($domain) {
    $this->response = FALSE;
    $this->output = NULL;
    $this->crypt = NULL;
    if (!$this->access($domain)) {
      return FALSE;
    }
    $this->remoteRequest();
    return $this->getResponse();
  }

  public function getResponse() {
    return $this->response;
  }

  /**
   * @param HttpRequest $request
   */
  protected function setRequestOptions($request) {}

  /**
   * @return bool
   */
  protected function processResponse() {
    return TRUE;
  }

  protected function remoteRequest() {
    /** @var DomainInterface $domain */
    $domain = $this->drdEntity;
    $class = explode('\\', get_class($this));

    // Add authentication
    $args = [
      'auth' => $domain->get('auth')->value,
      'authsetting' => $domain->getAuthSetting(),
      'action' => array_pop($class),
    ] + $this->arguments;

    // Encrypt the arguments
    $method = $domain->get('crypt')->value;
    if (empty($method)) {
      $this->log($domain, 'alert', 'No encryption configured yet.');
      return;
    }
    $this->crypt = \Drupal\drd\Crypt\Base::getInstance(
      $method,
      $domain->getCryptSetting()
    );

    $body = base64_encode(serialize([
      'uuid' => $domain->uuid(),
      'args' => $this->crypt->encrypt($args),
      'iv' => $this->crypt->getIV(),
    ]));

    /** @var HttpRequest $request */
    $request = \Drupal::service('drd.http_request');
    $request->setDomain($domain)
      ->setQuery('drd-agent')
      ->setOption('body', $body);
    $this->setRequestOptions($request);
    $request->request();
    if (!$request->isRemoteDrd()) {
      $this->log($domain, 'warning', 'Remote instance does not support DRD.');
      return;
    }
    $this->responseHeaders = $request->getResponseHeaders();
    if ($this->processResponse()) {
      try {
        $this->response = $this->crypt->decrypt($request->getResponse(), $this->crypt->getIV());
      } catch (\Exception $ex) {
        $this->log($domain, 'critical', 'Decrypt problem: @msg', ['@msg' => $ex->getMessage()]);
        return;
      }

      if (!empty($this->response['messages'])) {
        $domain->cacheRemoteMessages($this->response['messages']);
      }
      unset($this->response['messages']);
    }
    else {
      $this->response = TRUE;
    }
    $this->log($domain, 'info', 'Success with response: <pre>@response</pre>', ['@response' => print_r($this->response, TRUE)]);

    if ($followup = $this->getFollowUpAction()) {
      \Drupal::service('queue.drd')->createItem(Base::instance($followup), $domain);
    }
  }

}
